<?php
/***************************************************************
 *  Copyright notice
 *
 *  (c) 2009-2010 Frans van der Veen [Netcreators] <extensions@netcreators.com>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

namespace Netcreators\NcgovPdc\Service\Backend\Form;

use TYPO3\CMS\Backend\Form\FormEngine;
use TYPO3\CMS\Core\Utility\ExtensionManagementUtility;

/**
 * tce forms helper
 *
 * @author Frans van der Veen <extensions@netcreators.com>
 * @copyright Netcreators
 * @package NcgovPdc
 */
class MainFieldsPreProcessor
{

    protected $table = '';
    protected $row = '';
    protected $parent = '';

    protected $extensionConfiguration = '';

    /**
     * Initialize this object, so configuration values can be read.
     * @return void
     */
    public function initialize()
    {
        $this->extensionConfiguration = unserialize($GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf']['ncgov_pdc']);
    }

    /**
     * Preprocess selector
     * @param string $table
     * @param array $row
     * @param FormEngine $parent
     */
    function getMainFields_preProcess($table, array &$row, FormEngine &$parent)
    {
        $this->table = $table;
        $this->row = & $row;
        $this->parent = & $parent;
        switch ($table) {
            case 'tx_ncgovpdc_domain_model_frequentlyaskedquestionchannel':
                $this->initialize();
                $this->preProcessDomainModelFrequentlyAskedQuestionChannel();
                break;
            case 'tx_ncgovpdc_domain_model_frequentlyaskedquestion':
                $this->initialize();
                $this->preProcessDomainModelFrequentlyAskedQuestion();
                break;
            case 'tx_ncgovpdc_domain_model_product':
                $this->initialize();
                $this->preProcessDomainModelProduct();
                break;
            case 'tx_ncgovpdc_domain_model_referencelink':
                $this->preProcessReferenceLink();
                break;
            default:
                break;
        }
    }

    protected function preProcessReferenceLink()
    {
        if ($_GET['type'] && $_GET['subtype']) {
            $this->row['type'] = $_GET['type'];
            $this->row['subtype'] = $_GET['subtype'];
        }
    }

    /**
     * Preprocess faq table
     * @return void
     */
    protected function preProcessDomainModelFrequentlyAskedQuestionChannel()
    {
        $parentUid = $this->row['frequently_asked_question_uid'];
        $databaseResource = $GLOBALS['TYPO3_DB']->exec_selectQuery(
            '*',
            'tx_ncgovpdc_domain_model_frequentlyaskedquestion',
            'uid = ' . $parentUid
        );
        $row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($databaseResource);
        // ncfrans 2012-07-26 disabled, since antwoord does not exist anymore
//		if($row['imported']) {
//			// prevent the record from being saved
//			$this->addDoNotSaveCode($this->table);
//		}
    }

    // This could probably removed, but it's nice to keep this code as a reference.
    protected function addDoNotSaveCode($table)
    {
        $tempColumns = array(
            'dontsaveme' => array(
                'exclude' => 0,
                'label' => 'Dit record mag niet worden gewijzigd',
                'config' => array(
                    'type' => 'input',
                    'size' => 60,
                    'eval' => 'required,Netcreators\\NcgovPdc\\Service\\Backend\\Form\\FormEvals\\ImportedFrequentlyAskedQuestionReadonlyFormEval',
                ),
            ),
            'dontsaveme_alert' => array(
                'exclude' => 0,
                'label' => '-',
                'config' => array(
                    'type' => 'user',
                    'size' => 60,
                    'userFunc' => 'Netcreators\\NcgovPdc\\Service\\Backend\\Form\\FormEvals\\ImportedFrequentlyAskedQuestionReadonlyFormEval->alertNotAllowedToSave'
                ),
            ),
        );
        $GLOBALS['TCA'][$table]['columns']['dontsaveme'] = $tempColumns['dontsaveme'];
        $GLOBALS['TCA'][$table]['columns']['dontsaveme_alert'] = $tempColumns['dontsaveme_alert'];
        $GLOBALS['TCA'][$table]['types']['1']['showitem'] = 'dontsaveme,' . $GLOBALS['TCA'][$table]['types']['1']['showitem'] . ',dontsaveme_alert';
        ExtensionManagementUtility::addTCAcolumns($table, $tempColumns);
        ExtensionManagementUtility::addToAllTCAtypes($table, 'dontsaveme,dontsaveme_alert');
    }

    /**
     * Preprocess faq table
     * @return void
     */
    protected function preProcessDomainModelFrequentlyAskedQuestion()
    {
        // ncfrans 2012-07-26 disabled, since antwoord does not exist anymore
//		if($this->row['imported']) {
//			// prevent the record from being saved
//			$this->addDoNotSaveCode($this->table);
//		}
        $code = trim($this->extensionConfiguration['identifierUniqueCode']);
        if (empty($code)) {
            $code = '-----';
        }
        if (empty($this->row['owms_core_identifier'])) {
            $this->row['owms_core_identifier'] = $code . time();
        }
        if (empty($this->row['owms_core_modified'])) {
            $this->row['owms_core_modified'] = time();
        }
        if (empty($this->row['owms_core_creator_scheme'])) {
            $this->row['owms_core_creator_scheme'] = trim($this->extensionConfiguration['defaultCreatorScheme']);
        }
        if (empty($this->row['owms_core_creator'])) {
            $this->row['owms_core_creator'] = trim($this->extensionConfiguration['defaultCreator']);
        }
        if (empty($this->row['owms_core_spatial_scheme'])) {
            $this->row['owms_core_spatial_scheme'] = trim($this->extensionConfiguration['defaultSpatialScheme']);
        }
        if (empty($this->row['owms_core_spatial'])) {
            $this->row['owms_core_spatial'] = trim($this->extensionConfiguration['defaultSpatial']);
        }
        if (empty($this->row['owms_mantle_authority'])) {
            $this->row['owms_mantle_authority'] = trim($this->extensionConfiguration['defaultAuthority']);
        }
        if (empty($this->row['owms_mantle_contributor'])) {
            $this->row['owms_mantle_contributor'] = trim($this->extensionConfiguration['defaultContributor']);
        }
        if (empty($this->row['supplier_system'])) {
            $this->row['supplier_system'] = trim($this->extensionConfiguration['defaultSupplierSystem']);
        }
    }

    /**
     * Preprocess product table
     * @return void
     */
    protected function preProcessDomainModelProduct()
    {

        $code = trim($this->extensionConfiguration['identifierUniqueCode']);

        if (empty($code)) {
            $code = '-----';
        }

        if (empty($this->row['owms_core_identifier'])) {
            $this->row['owms_core_identifier'] = $code . time();
        }

        if (isset($this->extensionConfiguration['dontShowAnyRteOnRecordLoad']) && $this->extensionConfiguration['dontShowAnyRteOnRecordLoad']) {
            if (!isset($_POST['data'][$this->table])) {
                $this->row['show_fields'] = 0;
            }
        }

        if (isset($this->extensionConfiguration['showAllFields']) && $this->extensionConfiguration['showAllFields']) {
            $this->row['show_fields'] = 31;
        }

        $pageTsConfig = \TYPO3\CMS\Backend\Utility\BackendUtility::getPagesTSconfig($this->row['pid']);

        if (isset($pageTsConfig['tx_ncgovpdc.']) && isset($pageTsConfig['tx_ncgovpdc.']['alternativeShowItems'])) {
            $GLOBALS['TCA']['tx_ncgovpdc_domain_model_product']['types'][$this->row['show_fields']]['showitem'] = $pageTsConfig['tx_ncgovpdc.']['alternativeShowItems'];
        }

        if (isset($pageTsConfig['tx_ncgovpdc.']) && isset($pageTsConfig['tx_ncgovpdc.']['dynamicLabelsFromProductData.'])) {
            $dynamicLabelsFromProductData = $pageTsConfig['tx_ncgovpdc.']['dynamicLabelsFromProductData.'];

            if (count($dynamicLabelsFromProductData) > 0) {

                if (isset($dynamicLabelsFromProductData['defaultLabel']) && $dynamicLabelsFromProductData['defaultLabel'] != '') {
                    $defaultText = $dynamicLabelsFromProductData['defaultLabel'];
                }

                foreach ($dynamicLabelsFromProductData as $key => $searchReplaceConfiguration) {

                    if ($key == 'defaultLabel') {
                        continue;
                    }

                    if (isset($this->row[$searchReplaceConfiguration['replaceWithContentsOfColumn']]) && trim(
                            $this->row[$searchReplaceConfiguration['replaceWithContentsOfColumn']]
                        ) != ''
                    ) {
                        $replace = strip_tags($this->row[$searchReplaceConfiguration['replaceWithContentsOfColumn']]);

                        if (strlen($replace) > 26) {
                            $replace = substr($replace, 0, 26) . '...';
                        }

                    } else {
                        $replace = $defaultText;
                    }

                    $subject = $GLOBALS['TCA']['tx_ncgovpdc_domain_model_product']['types'][$this->row['show_fields']]['showitem'];
                    $search = $searchReplaceConfiguration['search'];
                    $subject = str_replace($search, $replace, $subject);
                    $GLOBALS['TCA']['tx_ncgovpdc_domain_model_product']['types'][$this->row['show_fields']]['showitem'] = $subject;
                }
            }
        }
    }
}

